@@ -748,9 +748,13 @@ mod tests {
                 ])))
             });
 
-        cx.cx
-            .cx
-            .simulate_mouse_move(screen_coord.unwrap(), None, Modifiers::command_shift());
+        let modifiers = if cfg!(target_os = "macos") {
+            Modifiers::command_shift()
+        } else {
+            Modifiers::control_shift()
+        };
+
+        cx.simulate_mouse_move(screen_coord.unwrap(), None, modifiers);
 
         requests.next().await;
         cx.run_until_parked();
@@ -767,9 +771,7 @@ mod tests {
             let variable = A;
         "});
 
-        cx.cx
-            .cx
-            .simulate_click(screen_coord.unwrap(), Modifiers::command_shift());
+        cx.simulate_click(screen_coord.unwrap(), modifiers);
 
         cx.assert_editor_state(indoc! {"
             struct «Aˇ»;
