@@ -154,12 +154,12 @@ mod tests {
         let cx = TestAppContext::new(dispatcher, None);
         cx.text_system()
             .add_fonts(vec![std::fs::read(
-                "../../assets/fonts/zed-mono/zed-mono-extended.ttf",
+                "../../assets/fonts/plex-mono/ZedPlexMono-Regular.ttf",
             )
             .unwrap()
             .into()])
             .unwrap();
-        let id = cx.text_system().font_id(&font("Zed Mono")).unwrap();
+        let id = cx.text_system().font_id(&font("Zed Plex Mono")).unwrap();
 
         cx.update(|cx| {
             let text_system = cx.text_system().clone();
