@@ -95,6 +95,10 @@ impl JsonLspAdapter {
                 "format": {
                     "enable": true,
                 },
+                "validate":
+                {
+                    "enable": true,
+                },
                 "schemas": [
                     {
                         "fileMatch": ["tsconfig.json"],
@@ -213,7 +217,12 @@ impl LspAdapter for JsonLspAdapter {
     }
 
     fn language_ids(&self) -> HashMap<String, String> {
-        [("JSON".into(), "jsonc".into())].into_iter().collect()
+        [
+            ("JSON".into(), "json".into()),
+            ("JSONC".into(), "jsonc".into()),
+        ]
+        .into_iter()
+        .collect()
     }
 }
 
