@@ -45,6 +45,7 @@ pub fn init(
         ("gowork", tree_sitter_gowork::language()),
         ("jsdoc", tree_sitter_jsdoc::language()),
         ("json", tree_sitter_json::language()),
+        ("jsonc", tree_sitter_json::language()),
         ("markdown", tree_sitter_markdown::language()),
         ("proto", tree_sitter_proto::language()),
         ("python", tree_sitter_python::language()),
@@ -126,6 +127,14 @@ pub fn init(
         ],
         json_task_context()
     );
+    language!(
+        "jsonc",
+        vec![Arc::new(json::JsonLspAdapter::new(
+            node_runtime.clone(),
+            languages.clone(),
+        ))],
+        json_task_context()
+    );
     language!("markdown");
     language!(
         "python",
