@@ -31,10 +31,11 @@ pub enum ComponentStory {
     OverflowScroll,
     Picker,
     Scroll,
+    Setting,
     Tab,
     TabBar,
     Text,
-    TitleBar,
+    // TitleBar,
     ToggleButton,
     ToolStrip,
     ViewportUnits,
@@ -64,10 +65,11 @@ impl ComponentStory {
             Self::ListItem => cx.new_view(|_| ui::ListItemStory).into(),
             Self::OverflowScroll => cx.new_view(|_| crate::stories::OverflowScrollStory).into(),
             Self::Scroll => ScrollStory::view(cx).into(),
+            Self::Setting => cx.new_view(|cx| ui::SettingStory::init(cx)).into(),
             Self::Text => TextStory::view(cx).into(),
             Self::Tab => cx.new_view(|_| ui::TabStory).into(),
             Self::TabBar => cx.new_view(|_| ui::TabBarStory).into(),
-            Self::TitleBar => cx.new_view(|_| ui::TitleBarStory).into(),
+            // Self::TitleBar => cx.new_view(|_| title_bar::TitleBarStory).into(),
             Self::ToggleButton => cx.new_view(|_| ui::ToggleButtonStory).into(),
             Self::ToolStrip => cx.new_view(|_| ui::ToolStripStory).into(),
             Self::ViewportUnits => cx.new_view(|_| crate::stories::ViewportUnitsStory).into(),
