@@ -64,7 +64,19 @@ use crate::mappings::{colors::to_alac_rgb, keys::to_esc_str};
 
 actions!(
     terminal,
-    [Clear, Copy, Paste, ShowCharacterPalette, SearchTest,]
+    [
+        Clear,
+        Copy,
+        Paste,
+        ShowCharacterPalette,
+        SearchTest,
+        ScrollLineUp,
+        ScrollLineDown,
+        ScrollPageUp,
+        ScrollPageDown,
+        ScrollToTop,
+        ScrollToBottom,
+    ]
 );
 
 ///Scrolling is unbearably sluggish by default. Alacritty supports a configurable
@@ -933,6 +945,18 @@ impl Terminal {
         &self.last_content
     }
 
+    pub fn total_lines(&self) -> usize {
+        let term = self.term.clone();
+        let terminal = term.lock_unfair();
+        terminal.total_lines()
+    }
+
+    pub fn viewport_lines(&self) -> usize {
+        let term = self.term.clone();
+        let terminal = term.lock_unfair();
+        terminal.screen_lines()
+    }
+
     //To test:
     //- Activate match on terminal (scrolling and selection)
     //- Editor search snapping behavior
@@ -982,6 +1006,46 @@ impl Terminal {
         self.events.push_back(InternalEvent::Clear)
     }
 
+    pub fn scroll_line_up(&mut self) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::Delta(1)));
+    }
+
+    pub fn scroll_up_by(&mut self, lines: usize) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::Delta(lines as i32)));
+    }
+
+    pub fn scroll_line_down(&mut self) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::Delta(-1)));
+    }
+
+    pub fn scroll_down_by(&mut self, lines: usize) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::Delta(-(lines as i32))));
+    }
+
+    pub fn scroll_page_up(&mut self) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::PageUp));
+    }
+
+    pub fn scroll_page_down(&mut self) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::PageDown));
+    }
+
+    pub fn scroll_to_top(&mut self) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::Top));
+    }
+
+    pub fn scroll_to_bottom(&mut self) {
+        self.events
+            .push_back(InternalEvent::Scroll(AlacScroll::Bottom));
+    }
+
     ///Resize the terminal and the PTY.
     pub fn set_size(&mut self, new_size: TerminalSize) {
         if self.last_content.size != new_size {
@@ -1394,6 +1458,13 @@ impl Terminal {
         })
     }
 
+    pub fn working_directory(&self) -> Option<PathBuf> {
+        self.pty_info
+            .current
+            .as_ref()
+            .map(|process| process.cwd.clone())
+    }
+
     pub fn title(&self, truncate: bool) -> String {
         const MAX_CHARS: usize = 25;
         match &self.task {
