@@ -4,6 +4,8 @@ mod checkbox;
 mod context_menu;
 mod disclosure;
 mod divider;
+mod dropdown_menu;
+mod facepile;
 mod icon;
 mod indicator;
 mod keybinding;
@@ -14,10 +16,10 @@ mod popover;
 mod popover_menu;
 mod radio;
 mod right_click_menu;
+mod setting;
 mod stack;
 mod tab;
 mod tab_bar;
-mod title_bar;
 mod tool_strip;
 mod tooltip;
 
@@ -30,6 +32,8 @@ pub use checkbox::*;
 pub use context_menu::*;
 pub use disclosure::*;
 pub use divider::*;
+use dropdown_menu::*;
+pub use facepile::*;
 pub use icon::*;
 pub use indicator::*;
 pub use keybinding::*;
@@ -40,10 +44,10 @@ pub use popover::*;
 pub use popover_menu::*;
 pub use radio::*;
 pub use right_click_menu::*;
+pub use setting::*;
 pub use stack::*;
 pub use tab::*;
 pub use tab_bar::*;
-pub use title_bar::*;
 pub use tool_strip::*;
 pub use tooltip::*;
 
