@@ -106,6 +106,7 @@ pub enum IconName {
     ChevronLeft,
     ChevronRight,
     ChevronUp,
+    ChevronUpDown,
     Close,
     Code,
     Collab,
@@ -141,13 +142,21 @@ pub enum IconName {
     Folder,
     FolderOpen,
     FolderX,
+    Font,
+    FontSize,
+    FontWeight,
     Github,
+    GenericMinimize,
+    GenericMaximize,
+    GenericClose,
+    GenericRestore,
     Hash,
     HistoryRerun,
     Indicator,
     IndicatorX,
     InlayHint,
     Library,
+    LineHeight,
     Link,
     ListTree,
     MagicWand,
@@ -181,8 +190,8 @@ pub enum IconName {
     RotateCw,
     Save,
     Screen,
-    SelectAll,
     SearchSelection,
+    SelectAll,
     Server,
     Settings,
     Shift,
@@ -212,6 +221,7 @@ pub enum IconName {
     ZedAssistant,
     ZedAssistantFilled,
     ZedXCopilot,
+    Visible,
 }
 
 impl IconName {
@@ -224,6 +234,7 @@ impl IconName {
             IconName::ArrowLeft => "icons/arrow_left.svg",
             IconName::ArrowRight => "icons/arrow_right.svg",
             IconName::ArrowUp => "icons/arrow_up.svg",
+            IconName::ArrowUpFromLine => "icons/arrow_up_from_line.svg",
             IconName::ArrowUpRight => "icons/arrow_up_right.svg",
             IconName::AtSign => "icons/at_sign.svg",
             IconName::AudioOff => "icons/speaker_off.svg",
@@ -243,6 +254,7 @@ impl IconName {
             IconName::ChevronLeft => "icons/chevron_left.svg",
             IconName::ChevronRight => "icons/chevron_right.svg",
             IconName::ChevronUp => "icons/chevron_up.svg",
+            IconName::ChevronUpDown => "icons/chevron_up_down.svg",
             IconName::Close => "icons/x.svg",
             IconName::Code => "icons/code.svg",
             IconName::Collab => "icons/user_group_16.svg",
@@ -278,13 +290,21 @@ impl IconName {
             IconName::Folder => "icons/file_icons/folder.svg",
             IconName::FolderOpen => "icons/file_icons/folder_open.svg",
             IconName::FolderX => "icons/stop_sharing.svg",
+            IconName::Font => "icons/font.svg",
+            IconName::FontSize => "icons/font_size.svg",
+            IconName::FontWeight => "icons/font_weight.svg",
             IconName::Github => "icons/github.svg",
+            IconName::GenericMinimize => "icons/generic_minimize.svg",
+            IconName::GenericMaximize => "icons/generic_maximize.svg",
+            IconName::GenericClose => "icons/generic_close.svg",
+            IconName::GenericRestore => "icons/generic_restore.svg",
             IconName::Hash => "icons/hash.svg",
             IconName::HistoryRerun => "icons/history_rerun.svg",
             IconName::Indicator => "icons/indicator.svg",
             IconName::IndicatorX => "icons/indicator_x.svg",
             IconName::InlayHint => "icons/inlay_hint.svg",
             IconName::Library => "icons/library.svg",
+            IconName::LineHeight => "icons/line_height.svg",
             IconName::Link => "icons/link.svg",
             IconName::ListTree => "icons/list_tree.svg",
             IconName::MagicWand => "icons/magic_wand.svg",
@@ -308,18 +328,18 @@ impl IconName {
             IconName::Quote => "icons/quote.svg",
             IconName::Regex => "icons/regex.svg",
             IconName::Replace => "icons/replace.svg",
-            IconName::Reveal => "icons/reveal.svg",
             IconName::ReplaceAll => "icons/replace_all.svg",
             IconName::ReplaceNext => "icons/replace_next.svg",
             IconName::ReplyArrowRight => "icons/reply_arrow_right.svg",
             IconName::Rerun => "icons/rerun.svg",
             IconName::Return => "icons/return.svg",
-            IconName::RotateCw => "icons/rotate_cw.svg",
+            IconName::Reveal => "icons/reveal.svg",
             IconName::RotateCcw => "icons/rotate_ccw.svg",
+            IconName::RotateCw => "icons/rotate_cw.svg",
             IconName::Save => "icons/save.svg",
             IconName::Screen => "icons/desktop.svg",
-            IconName::SelectAll => "icons/select_all.svg",
             IconName::SearchSelection => "icons/search_selection.svg",
+            IconName::SelectAll => "icons/select_all.svg",
             IconName::Server => "icons/server.svg",
             IconName::Settings => "icons/file_icons/settings.svg",
             IconName::Shift => "icons/shift.svg",
@@ -349,7 +369,7 @@ impl IconName {
             IconName::ZedAssistant => "icons/zed_assistant.svg",
             IconName::ZedAssistantFilled => "icons/zed_assistant_filled.svg",
             IconName::ZedXCopilot => "icons/zed_x_copilot.svg",
-            IconName::ArrowUpFromLine => "icons/arrow_up_from_line.svg",
+            IconName::Visible => "icons/visible.svg",
         }
     }
 }
