@@ -10,9 +10,9 @@ mod label;
 mod list;
 mod list_header;
 mod list_item;
+mod setting;
 mod tab;
 mod tab_bar;
-mod title_bar;
 mod toggle_button;
 mod tool_strip;
 
@@ -28,8 +28,8 @@ pub use label::*;
 pub use list::*;
 pub use list_header::*;
 pub use list_item::*;
+pub use setting::*;
 pub use tab::*;
 pub use tab_bar::*;
-pub use title_bar::*;
 pub use toggle_button::*;
 pub use tool_strip::*;
