@@ -71,6 +71,16 @@ pub trait SearchableItem: Item + EventEmitter<SearchEvent> {
     fn activate_match(&mut self, index: usize, matches: &[Self::Match], cx: &mut ViewContext<Self>);
     fn select_matches(&mut self, matches: &[Self::Match], cx: &mut ViewContext<Self>);
     fn replace(&mut self, _: &Self::Match, _: &SearchQuery, _: &mut ViewContext<Self>);
+    fn replace_all(
+        &mut self,
+        matches: &mut dyn Iterator<Item = &Self::Match>,
+        query: &SearchQuery,
+        cx: &mut ViewContext<Self>,
+    ) {
+        for item in matches {
+            self.replace(item, query, cx);
+        }
+    }
     fn match_index_for_direction(
         &mut self,
         matches: &[Self::Match],
@@ -123,6 +133,12 @@ pub trait SearchableItemHandle: ItemHandle {
         _: &SearchQuery,
         _: &mut WindowContext,
     );
+    fn replace_all(
+        &self,
+        matches: &mut dyn Iterator<Item = any_vec::element::ElementRef<'_, dyn Send>>,
+        query: &SearchQuery,
+        cx: &mut WindowContext,
+    );
     fn match_index_for_direction(
         &self,
         matches: &AnyVec<dyn Send>,
@@ -241,6 +257,17 @@ impl<T: SearchableItem> SearchableItemHandle for View<T> {
         self.update(cx, |this, cx| this.replace(mat, query, cx))
     }
 
+    fn replace_all(
+        &self,
+        matches: &mut dyn Iterator<Item = any_vec::element::ElementRef<'_, dyn Send>>,
+        query: &SearchQuery,
+        cx: &mut WindowContext,
+    ) {
+        self.update(cx, |this, cx| {
+            this.replace_all(&mut matches.map(|m| m.downcast_ref().unwrap()), query, cx);
+        })
+    }
+
     fn search_bar_visibility_changed(&self, visible: bool, cx: &mut WindowContext) {
         self.update(cx, |this, cx| {
             this.search_bar_visibility_changed(visible, cx)
